@@ -160,7 +160,7 @@ class DialogMirror(QDialog):
         afterwards in the treeview '''
     class MirrorTestKDE(MirrorTest):
         def __init__(self, mirrors, test_file, running, dialog, parent):
-            MirrorTest.__init__(self, mirrors, test_file, running)
+            MirrorTest.__init__(self, mirrors, test_file, None, running)
             self.dialog = dialog
             self.parent = parent
 
