@@ -25,6 +25,7 @@ import re
 import os
 import subprocess
 import tempfile
+import shutil
 
 from threading import Thread
 import pycurl
@@ -52,7 +53,7 @@ def expand_ppa_line(abrev, distro_codename):
         ppa_name = abrev.split("/")[1]
     except IndexError, e:
         ppa_name = "ppa"
-    sourceslistd = apt_pkg.Config.find_dir("Dir::Etc::sourceparts")
+    sourceslistd = apt_pkg.config.find_dir("Dir::Etc::sourceparts")
     line = "deb http://ppa.launchpad.net/%s/%s/ubuntu %s main" % (
         ppa_owner, ppa_name, distro_codename)
     filename = "%s/%s-%s-%s.list" % (
@@ -115,7 +116,7 @@ class AddPPASigningKeyThread(Thread):
         try:
             ppa_info = get_ppa_info_from_lp(owner_name, ppa_name)
         except pycurl.error as e:
-            print "Error reading %s: %s" % (lp_url, e[1])
+            print "Error reading %s: %s" % (ppa_path, e[1])
             return False
         try:
             signing_key_fingerprint = ppa_info["signing_key_fingerprint"]
